import { Type } from '@sinclair/typebox';
import { DeviceShadow } from './DeviceShadow.js';
import { toFetchError } from './FetchError.js';
import { slashless } from './slashless.js';
import { validatedFetch } from './validatedFetch.js';
const Page = (Item) => Type.Object({
    total: Type.Integer(),
    items: Type.Array(Item),
});
const Devices = Page(DeviceShadow);
/**
 * @link https://api.nrfcloud.com/v1/#tag/IP-Devices/operation/ProvisionDevices
 */
const ProvisionDevice = Type.Object({
    bulkOpsRequestId: Type.String(),
});
/**
 * firmware types supported by a device for FOTA
 */
export var FwType;
(function (FwType) {
    FwType["APP"] = "APP";
    FwType["MODEM"] = "MODEM";
    FwType["BOOT"] = "BOOT";
    FwType["SOFTDEVICE"] = "SOFTDEVICE";
    FwType["BOOTLOADER"] = "BOOTLOADER";
    FwType["MDM_FULL"] = "MDM_FULL";
})(FwType || (FwType = {}));
export const devices = ({ endpoint, apiKey, }, fetchImplementation) => {
    const headers = {
        Authorization: `Bearer ${apiKey}`,
        Accept: 'application/json; charset=utf-8',
    };
    const vf = validatedFetch({ endpoint, apiKey }, fetchImplementation);
    return {
        // FIXME: implement pagination
        list: async () => vf({
            resource: `devices?${new URLSearchParams({
                pageLimit: '100',
                deviceNameFuzzy: 'oob-',
            }).toString()}`,
        }, Devices),
        get: async (id) => vf({ resource: `devices/${encodeURIComponent(id)}` }, DeviceShadow),
        updateState: async (id, state) => fetch(`${slashless(endpoint)}/v1/devices/${encodeURIComponent(id)}/state`, {
            headers: {
                ...headers,
                'Content-Type': 'application/json',
            },
            method: 'PATCH',
            body: JSON.stringify(state),
        }).then(async (res) => {
            if (res.status >= 400)
                return { error: await toFetchError(res) };
            return { success: true };
        }),
        register: async (devices) => {
            const bulkRegistrationPayload = devices
                .map(({ deviceId, subType, tags, fwTypes, certPem }) => {
                const deviceFwTypes = fwTypes ?? Object.values(FwType);
                return [
                    [
                        deviceId,
                        subType ?? '',
                        (tags ?? []).join('|'),
                        deviceFwTypes.join('|'),
                        `"${certPem}"`,
                    ],
                ];
            })
                .map((cols) => cols.join(','))
                .join('\n');
            const maybeResult = await vf({
                resource: 'devices',
                payload: {
                    body: bulkRegistrationPayload,
                    type: 'application/octet-stream',
                },
            }, ProvisionDevice);
            if ('error' in maybeResult)
                return maybeResult;
            return { bulkOpsRequestId: maybeResult.result.bulkOpsRequestId };
        },
        remove: async (deviceId) => {
            const maybeResult = await vf({
                method: 'DELETE',
                resource: `devices/${deviceId}`,
            }, Type.Any());
            if ('error' in maybeResult)
                return maybeResult;
            return { success: true };
        },
    };
};
